Package: LegATo
Title: LegATo: Longitudinal mEtaGenomic Analysis Toolkit
Version: 1.5.0
Authors@R: c(
    person("Aubrey", "Odom", , "aodom@bu.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7113-7598")),
    person("Yilong", "Zhang", , "elong0527@gmail.com", role = "ctb",
           comment = "Author of NMIT functions"),
    person("Jared", "Pincus", , "pincus@bu.edu", role = "csl",
           comment = c(ORCID = "0000-0001-6708-5262", other = "Naming consultant of package")),
    person("Jordan", "Pincus", , "pincus@bu.edu", role = "art",
           comment = "Artist of LegATo icon")
  )
Description: LegATo is a suite of open-source software tools for
    longitudinal microbiome analysis. It is extendable to several
    different study forms with optimal ease-of-use for researchers.
    Microbiome time-series data presents distinct challenges including
    complex covariate dependencies and variety of longitudinal study
    designs. This toolkit will allow researchers to determine which
    microbial taxa are affected over time by perturbations such as onset
    of disease or lifestyle choices, and to predict the effects of these
    perturbations over time, including changes in composition or stability
    of commensal bacteria.
License: MIT + file LICENSE
URL: https://wejlab.github.io/LegATo-docs/
BugReports: https://github.com/wejlab/LegATo/issues
Depends: R (>= 4.4.0)
Imports: animalcules, data.table, dplyr, ggplot2, magrittr,
        MultiAssayExperiment, plyr, rlang, S4Vectors, stringr,
        SummarizedExperiment, tibble, tidyr, tidyselect
Suggests: BiocStyle, broom, broom.mixed, circlize, ComplexHeatmap,
        emmeans, geepack, ggalluvial, ggeffects, grDevices, knitr,
        lme4, lmerTest, methods, RColorBrewer, rmarkdown, spelling,
        TBSignatureProfiler, testthat (>= 3.0.0),
        TreeSummarizedExperiment, usethis, vegan
VignetteBuilder: knitr
BiocType: Software
biocViews: MicrobiomeData, ReproducibleResearch, SequencingData
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/LegATo
git_branch: devel
git_last_commit: e6be75b
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 16:29:07 UTC; biocbuild
Author: Aubrey Odom [aut, cre] (ORCID: <https://orcid.org/0000-0001-7113-7598>),
  Yilong Zhang [ctb] (Author of NMIT functions),
  Jared Pincus [csl] (ORCID: <https://orcid.org/0000-0001-6708-5262>,
    other: Naming consultant of package),
  Jordan Pincus [art] (Artist of LegATo icon)
Maintainer: Aubrey Odom <aodom@bu.edu>
